@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -605,6 +605,28 @@ void gui_execute_action_id (gGui_t *gui, action_id_t action) {
     panel_toggle_visibility (NULL, gui->panel);
     break;
 
+  case ACTID_FLIP_V:
+  case ACTID_FLIP_H:
+#ifdef XINE_VO_TRANSFORM_FLIP_H
+    {
+      int flags = gui->transform.flags ^ (action == ACTID_FLIP_H ? 1 : 2);
+
+      flags = ((flags & 1) ? XINE_VO_TRANSFORM_FLIP_H : 0)
+            | ((flags & 2) ? XINE_VO_TRANSFORM_FLIP_V : 0);
+      xine_set_param (gui->stream, XINE_PARAM_VO_TRANSFORM, flags);
+      flags = xine_get_param (gui->stream, XINE_PARAM_VO_TRANSFORM);
+      if (flags != -1) {
+        flags = ((flags & XINE_VO_TRANSFORM_FLIP_H) ? 1 : 0)
+              | ((flags & XINE_VO_TRANSFORM_FLIP_V) ? 2 : 0);
+        if (flags != gui->transform.flags) {
+          gui->transform.flags = flags;
+          osd_display_info (gui, "%s", gui->transform.msg[flags & 3]);
+        }
+      }
+    }
+#endif
+    break;
+
   case ACTID_TOGGLE_FULLSCREEN:
     {
       int yes = numeric_set
@@ -1134,8 +1156,8 @@ void gui_execute_action_id (gGui_t *gui, action_id_t action) {
 
   case ACTID_PLAYLIST_OPEN:
     if (sarg) {
-        mediamark_load_mediamarks (gui, sarg);
-        gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+        gui_playlist_add_item (gui, sarg, 1, GUI_ITEM_TYPE_PLAYLIST, 1);
+        gui_current_set_index (gui, GUI_MMK_CURRENT);
         playlist_update_playlist (gui);
         if ((!is_playback_widgets_enabled (gui->panel)) && gui->playlist.num)
           enable_playback_controls (gui->panel, 1);
@@ -1194,16 +1216,16 @@ int gui_playlist_play (gGui_t *gui, int idx) {
 
   if(idx >= gui->playlist.num)
     return 0;
-  gui_set_current_mmk_by_index (gui, idx);
+  gui_current_set_index (gui, idx);
 
-  pthread_mutex_lock(&gui->mmk_mutex);
+  gui_playlist_lock (gui);
   if (!gui_xine_open_and_play (gui, gui->mmk.mrl, gui->mmk.sub, 0,
     gui->mmk.start, gui->mmk.av_offset, gui->mmk.spu_offset,
     !mediamark_have_alternates (&(gui->mmk))) &&
      (!mediamark_have_alternates(&(gui->mmk)) ||
         !gui_open_and_play_alternates (gui, &(gui->mmk), gui->mmk.sub)))
     ret = 0;
-  pthread_mutex_unlock(&gui->mmk_mutex);
+  gui_playlist_unlock (gui);
   return ret;
 }
 
@@ -1226,7 +1248,7 @@ void gui_playlist_start_next (gGui_t *gui) {
   }
 
   if (is_playback_widgets_enabled (gui->panel) && (!gui->playlist.num)) {
-    gui_set_current_mmk (gui, NULL);
+    gui_current_set_index (gui, GUI_MMK_NONE);
     enable_playback_controls (gui->panel, 0);
     gui_display_logo (gui);
     return;
@@ -1339,35 +1361,9 @@ void gui_init (gGui_t *gui, gui_init_params_t *p) {
    * init playlist
    */
   for (i = 0; i < p->num_files; i++) {
-    char *file = p->filenames[i];
+    const char *file = p->filenames[i];
 
-    /* grab recursively all files from dir */
-    if(is_a_dir(file)) {
-
-      if(file[strlen(file) - 1] == '/')
-	file[strlen(file) - 1] = '\0';
-
-      mediamark_collect_from_directory (gui, file);
-    }
-    else {
-
-      if(mrl_look_like_playlist(file))
-	(void) mediamark_concat_mediamarks (gui, file);
-      else {
-	char *sub = NULL;
-
-	if((sub = (char *)get_last_double_semicolon(file)) != NULL) {
-	  if(is_ipv6_last_double_semicolon(file))
-	    sub = NULL;
-	  else {
-	    *sub = 0;
-	    sub += 2;
-	  }
-	}
-
-        mediamark_append_entry (gui, (const char *)file, (const char *)file, sub, 0, -1, 0, 0);
-      }
-    }
+    gui_playlist_add_item (gui, file, GUI_MAX_DIR_LEVELS, GUI_ITEM_TYPE_AUTO, 0);
   }
 
   gui->playlist.cur = gui->playlist.num ? 0 : -1;
@@ -1384,6 +1380,12 @@ void gui_init (gGui_t *gui, gui_init_params_t *p) {
    *
    */
 
+  gui->transform.msg[0] = _("Video upright");
+  gui->transform.msg[1] = _("Video mirrored");
+  gui->transform.msg[2] = _("Video upside down");
+  gui->transform.msg[3] = _("Video upside down and mirrored");
+  gui->transform.flags = 0;
+
   use_x_lock_display =
     xine_config_register_bool (gui->xine, "gui.use_XLockDisplay", 1,
         _("Enable extra XLib thread safety."),
@@ -1507,7 +1509,7 @@ void gui_init (gGui_t *gui, gui_init_params_t *p) {
     xine_config_register_bool (gui->xine, "gui.osd_enabled",
 			      1,
 			      _("Enable OSD support"),
-			      _("Enabling OSD permit to display some status/informations "
+			      _("Enabling OSD permit one to display some status/informations "
 				"in output window."),
 			      CONFIG_LEVEL_BEG,
 			      osd_enabled_cb,
@@ -1644,7 +1646,7 @@ void gui_init (gGui_t *gui, gui_init_params_t *p) {
   gui->kbindings = kbindings_init_kbinding (gui, gui->keymap_file);
   gui->kbindings_enabled = !!gui->kbindings;
 
-  gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+  gui_current_set_index (gui, GUI_MMK_CURRENT);
 
   panel_init (gui);
   gui->event_reject = 0;
@@ -1727,11 +1729,11 @@ void gui_run(gGui_t *gui, char **session_opts) {
 
 	  if(autoplay_mrls) {
 	    for (j = 0; j < num_mrls; j++)
-                mediamark_append_entry (gui, autoplay_mrls[j],
+                gui_playlist_append (gui, autoplay_mrls[j],
 				     autoplay_mrls[j], NULL, 0, -1, 0, 0);
 
 	    gui->playlist.cur = 0;
-            gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+            gui_current_set_index (gui, GUI_MMK_CURRENT);
 	  }
 	}
       }
@@ -1804,7 +1806,7 @@ void gui_run(gGui_t *gui, char **session_opts) {
 
     /* User load a playlist on startup */
     if(actions_on_start(gui->actions_on_start, ACTID_PLAYLIST)) {
-      gui_set_current_mmk (gui, mediamark_get_current_mmk (gui));
+      gui_current_set_index (gui, GUI_MMK_CURRENT);
 
       if(gui->playlist.num) {
 	gui->playlist.cur = 0;
@@ -1846,7 +1848,7 @@ void gui_run(gGui_t *gui, char **session_opts) {
     char buffer[XITK_PATH_MAX + XITK_NAME_MAX + 1];
 
     snprintf(buffer, sizeof(buffer), "%s/.xine/xine-ui_old_playlist.tox", xine_get_homedir());
-    mediamark_save_mediamarks (gui, buffer);
+    gui_playlist_save (gui, buffer);
   }
 
   gui->running = 0;
