@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a unix video player.
  *
@@ -49,8 +49,6 @@ typedef struct _slider_private_s {
   xitk_part_image_t       bg_skin;
 
   xitk_state_callback_t   motion_callback;
-  void                   *motion_userdata;
-
   xitk_state_callback_t   callback;
   void                   *userdata;
 
@@ -616,7 +614,7 @@ static int _xitk_slider_click (_slider_private_t *wp, int button, int bUp, int x
         }
         /* Exec motion callback function (if available) */
         if (moved && wp->motion_callback)
-          wp->motion_callback (&wp->w, wp->motion_userdata, value);
+          wp->motion_callback (&wp->w, wp->userdata, value);
       } else {
         if (moved || (wp->w.state & XITK_WIDGET_STATE_CLICK)) {
           wp->w.state &= ~XITK_WIDGET_STATE_CLICK;
@@ -734,7 +732,7 @@ static int _xitk_slider_key (_slider_private_t *wp, const char *string, int modi
       if (wp->callback)
         wp->callback (&wp->w, wp->userdata, nv);
       else if (wp->motion_callback)
-        wp->motion_callback (&wp->w, wp->motion_userdata, nv);
+        wp->motion_callback (&wp->w, wp->userdata, nv);
     }
     return 1;
   }
@@ -781,7 +779,7 @@ static int _xitk_slider_key (_slider_private_t *wp, const char *string, int modi
       if (wp->callback)
         wp->callback (&wp->w, wp->userdata, v);
       else if (wp->motion_callback)
-        wp->motion_callback (&wp->w, wp->motion_userdata, v);
+        wp->motion_callback (&wp->w, wp->userdata, v);
     }
     return 1;
   }
@@ -967,9 +965,8 @@ static xitk_widget_t *_xitk_slider_create (_slider_private_t *wp, const xitk_sli
   wp->paddle_drawn_here.fx = -1;
 
   wp->motion_callback    = s->motion_callback;
-  wp->motion_userdata    = s->motion_userdata;
   wp->callback           = s->callback;
-  wp->userdata           = s->userdata;
+  wp->userdata           = s->nw.userdata;
 
   wp->w.width            = wp->bg_skin.width;
   wp->w.height           = wp->bg_skin.height;
@@ -983,18 +980,16 @@ static xitk_widget_t *_xitk_slider_create (_slider_private_t *wp, const xitk_sli
 /*
  * Create the widget
  */
-xitk_widget_t *xitk_slider_create(xitk_widget_list_t *wl,
-    xitk_skin_config_t *skonfig, const xitk_slider_widget_t *s) {
+xitk_widget_t *xitk_slider_create (const xitk_slider_widget_t *s, xitk_skin_config_t *skonfig) {
   _slider_private_t *wp;
 
-  ABORT_IF_NULL(wl);
-  XITK_CHECK_CONSTITENCY(s);
+  _xitk_new_widget_check (&s->nw);
 
-  wp = (_slider_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  wp = (_slider_private_t *)xitk_widget_new (s->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
   strlcpy (wp->skin_element_name,
-    s->skin_element_name && s->skin_element_name[0] ? s->skin_element_name : "-",
+    s->nw.skin_element_name && s->nw.skin_element_name[0] ? s->nw.skin_element_name : "-",
     sizeof (wp->skin_element_name));
   _xitk_slider_set_skin (wp, skonfig);
 
@@ -1004,34 +999,32 @@ xitk_widget_t *xitk_slider_create(xitk_widget_list_t *wl,
 /*
  * Create the widget (not skined).
  */
-xitk_widget_t *xitk_noskin_slider_create (xitk_widget_list_t *wl,
-    const xitk_slider_widget_t *s, int x, int y, int width, int height, int type) {
+xitk_widget_t *xitk_noskin_slider_create (const xitk_slider_widget_t *s,
+  int x, int y, int width, int height, int type) {
   _slider_private_t *wp;
 
-  ABORT_IF_NULL(wl);
+  _xitk_new_widget_check (&s->nw);
 
-  XITK_CHECK_CONSTITENCY(s);
-
-  wp = (_slider_private_t *)xitk_widget_new (wl, sizeof (*wp));
+  wp = (_slider_private_t *)xitk_widget_new (s->nw.wl, sizeof (*wp));
   if (!wp)
     return NULL;
 
   wp->skin_element_name[0] = 0;
 
   if (type == XITK_VSLIDER) {
-    wp->paddle_skin.image = xitk_image_new (wl->xitk, NULL, 0, width * 3, height / 5);
+    wp->paddle_skin.image = xitk_image_new (wp->w.wl->xitk, NULL, 0, width * 3, height / 5);
     xitk_image_draw_paddle_three_state_vertical (wp->paddle_skin.image);
   } else if (type == XITK_HSLIDER) {
-    wp->paddle_skin.image = xitk_image_new (wl->xitk, NULL, 0, (width / 5) * 3, height);
+    wp->paddle_skin.image = xitk_image_new (wp->w.wl->xitk, NULL, 0, (width / 5) * 3, height);
     xitk_image_draw_paddle_three_state_horizontal (wp->paddle_skin.image);
   } else if (type == XITK_RSLIDER) {
     int w;
 
     w = ((((width + height) >> 1) >> 1) / 10) * 3;
-    wp->paddle_skin.image = xitk_image_new (wl->xitk, NULL, 0, (w * 3), w);
+    wp->paddle_skin.image = xitk_image_new (wp->w.wl->xitk, NULL, 0, (w * 3), w);
     xitk_image_draw_paddle_rotate (wp->paddle_skin.image);
   } else if (type == XITK_HVSLIDER) {
-    wp->paddle_skin.image = xitk_image_new (wl->xitk, NULL, 0, width * 3, height);
+    wp->paddle_skin.image = xitk_image_new (wp->w.wl->xitk, NULL, 0, width * 3, height);
     /* defer init to xitk_slider_hv_sync (). */
   } else {
     XITK_DIE ("Slider type unhandled.\n");
@@ -1041,7 +1034,7 @@ xitk_widget_t *xitk_noskin_slider_create (xitk_widget_list_t *wl,
   wp->paddle_skin.width = wp->paddle_skin.image->width;
   wp->paddle_skin.height = wp->paddle_skin.image->height;
 
-  wp->bg_skin.image = xitk_image_new (wl->xitk, NULL, 0, width, height);
+  wp->bg_skin.image = xitk_image_new (wp->w.wl->xitk, NULL, 0, width, height);
   if ((type == XITK_HSLIDER) || (type == XITK_VSLIDER) || (type == XITK_HVSLIDER))
     xitk_image_draw_inner (wp->bg_skin.image, width, height);
   else if (type == XITK_RSLIDER) {
